@@ -4,7 +4,7 @@
    */
 
 /*
- * Copyright 2007-2009 Roy Marples <roy@marples.name>
+ * Copyright (c) 2007-2009 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -78,8 +78,10 @@ _rc_can_find_pids(void)
 }
 
 static void
-print_level(const char *level)
+print_level(const char *prefix, const char *level)
 {
+	if (prefix)
+		printf("%s ", prefix);
 	printf ("Runlevel: ");
 	if (isatty(fileno(stdout)))
 		printf("%s%s%s\n",
@@ -172,9 +174,10 @@ print_services(const char *runlevel, RC_STRINGLIST *svcs)
 
 #include "_usage.h"
 #define extraopts "[runlevel1] [runlevel2] ..."
-#define getoptstring "alrsu" getoptstring_COMMON
+#define getoptstring "aclrsu" getoptstring_COMMON
 static const struct option longopts[] = {
 	{"all",         0, NULL, 'a'},
+	{"crashed",     0, NULL, 'c'},
 	{"list",        0, NULL, 'l'},
 	{"runlevel",    0, NULL, 'r'},
 	{"servicelist", 0, NULL, 's'},
@@ -183,6 +186,7 @@ static const struct option longopts[] = {
 };
 static const char * const longopts_help[] = {
 	"Show services from all run levels",
+	"Show crashed services",
 	"Show list of run levels",
 	"Show the name of the current runlevel",
 	"Show service list",
@@ -196,7 +200,7 @@ rc_status(int argc, char **argv)
 {
 	RC_STRING *s, *l, *t;
 	char *p, *runlevel = NULL;
-	int opt, aflag = 0;
+	int opt, aflag = 0, retval = 0;
 
 	test_crashed = _rc_can_find_pids();
 
@@ -207,12 +211,21 @@ rc_status(int argc, char **argv)
 			aflag++;
 			levels = rc_runlevel_list();
 			break;
-		case 'l':
-			levels = rc_runlevel_list();
-			TAILQ_FOREACH (l, levels, entries)
-				printf("%s\n", l->value);
+		case 'c':
+			services = rc_services_in_state(RC_SERVICE_STARTED);
+			retval = 1;
+			TAILQ_FOREACH(s, services, entries)
+				if (rc_service_daemons_crashed(s->value)) {
+					printf("%s\n", s->value);
+					retval = 0;
+				}
 			goto exit;
 			/* NOTREACHED */
+		case 'l':
+			levels = rc_runlevel_list();
+			TAILQ_FOREACH(l, levels, entries)
+				printf("%s\n", l->value);
+			goto exit;
 		case 'r':
 			runlevel = rc_runlevel_get();
 			printf("%s\n", runlevel);
@@ -263,16 +276,28 @@ rc_status(int argc, char **argv)
 	deptree = _rc_deptree_load(0, NULL);
 
 	TAILQ_FOREACH(l, levels, entries) {
-		print_level(l->value);
+		print_level(NULL, l->value);
 		services = rc_services_in_runlevel(l->value);
 		print_services(l->value, services);
+		nservices = rc_runlevel_stacks(l->value);
+		TAILQ_FOREACH(s, nservices, entries) {
+			if (rc_stringlist_find(levels, s->value) != NULL)
+				continue;
+			print_level("Stacked", s->value);
+			sservices = rc_services_in_runlevel(s->value);
+			print_services(s->value, sservices);
+			rc_stringlist_free(sservices);
+		}
+		sservices = NULL;
+		rc_stringlist_free(nservices);
+		nservices = NULL;
 		rc_stringlist_free(services);
 		services = NULL;
 	}
 
 	if (aflag || argc < 2) {
 		/* Show hotplugged services */
-		print_level("hotplugged");
+		print_level("Dynamic", "hotplugged");
 		services = rc_services_in_state(RC_SERVICE_HOTPLUGGED);
 		print_services(NULL, services);
 		rc_stringlist_free(services);
@@ -297,9 +322,10 @@ rc_status(int argc, char **argv)
 		}
 		TAILQ_FOREACH_SAFE(s, services, entries, t) {
 			if (rc_stringlist_find(sservices, s->value) ||
-			    rc_service_state(s->value) & RC_SERVICE_STOPPED)
-			{
-				TAILQ_REMOVE(services, s, entries);
+			    rc_service_state(s->value) &
+			    (RC_SERVICE_STOPPED | RC_SERVICE_HOTPLUGGED))
+		{
+			TAILQ_REMOVE(services, s, entries);
 				free(s->value);
 				free(s);
 			}
@@ -324,9 +350,9 @@ rc_status(int argc, char **argv)
 			rc_stringlist_free(tmp);
 		}
 		l->value = p;
-		print_level("needed");
+		print_level("Dynamic", "needed");
 		print_services(NULL, nservices);
-		print_level("manual");
+		print_level("Dynamic", "manual");
 		print_services(NULL, services);
 	}
 
@@ -343,5 +369,5 @@ exit:
 	rc_deptree_free(deptree);
 #endif
 
-	return(EXIT_SUCCESS);
+	return retval;
 }
