@@ -43,7 +43,7 @@
 #include <sys/ioctl.h>
 #include <sys/resource.h>
 #include <sys/stat.h>
-#include <sys/termios.h>
+#include <termios.h>
 #include <sys/time.h>
 #include <sys/wait.h>
 
@@ -322,7 +322,7 @@ get_pid(const char *pidfile, bool quiet)
 		fclose(fp);
 		return -1;
 	}
-	
+
 	fclose(fp);
 
 	return pid;
@@ -631,7 +631,7 @@ static const struct option longopts[] = {
 	longopts_COMMON
 };
 static const char * const longopts_help[] = {
-	"Set an ionice class:data when starting", 
+	"Set an ionice class:data when starting",
 	"Stop daemon",
 	"Set a nicelevel when starting",
 	"Retry schedule to use when stopping",
@@ -1210,7 +1210,7 @@ start_stop_daemon(int argc, char **argv)
 		/* Clean the environment of any RC_ variables */
 		env_list = rc_stringlist_new();
 		i = 0;
-		while(environ[i])
+		while (environ[i])
 			rc_stringlist_add(env_list, environ[i++]);
 
 #ifdef HAVE_PAM
@@ -1343,7 +1343,7 @@ start_stop_daemon(int argc, char **argv)
 	if (start_wait > 0) {
 		struct timespec ts;
 		bool alive = false;
-		
+
 		ts.tv_sec = start_wait / 1000;
 		ts.tv_nsec = (start_wait % 1000) * ONE_MS;
 		if (nanosleep(&ts, NULL) == -1) {
